@@ -6,8 +6,11 @@ from PyQt6.QtCore import Qt, QPropertyAnimation, QRect, QSize, QEvent, QEasingCu
 from PyQt6.QtGui import QResizeEvent, QIcon
 from PyQt6.QtWidgets import QWidget, QVBoxLayout, QFrame, QApplication
 
-from .navigation_widget import NavigationPushButton, NavigationToolButton, NavigationWidget, NavigationSeparator
-from ..widgets.scroll_area import ScrollArea
+from .navigation_widget import (NavigationTreeWidgetBase, NavigationToolButton, NavigationWidget, NavigationSeparator,
+                                NavigationTreeWidget)
+from ..widgets.scroll_area import SingleDirectionScrollArea
+from ..widgets.tool_tip import ToolTipFilter
+from ...common.router import qrouter
 from ...common.style_sheet import FluentStyleSheet
 from ...common.icon import FluentIconBase
 from ...common.icon import FluentIcon as FIF
@@ -21,13 +24,35 @@ class NavigationDisplayMode(Enum):
     MENU = 3
 
 
-class NavigationItemPostion(Enum):
+class NavigationItemPosition(Enum):
     """ Navigation item position """
     TOP = 0
     SCROLL = 1
     BOTTOM = 2
 
 
+class NavigationToolTipFilter(ToolTipFilter):
+    """ Navigation tool tip filter """
+
+    def _canShowToolTip(self) -> bool:
+        isVisible = super()._canShowToolTip()
+        parent = self.parent()  # type: NavigationWidget
+        return isVisible and parent.isCompacted
+
+
+class RouteKeyError(Exception):
+    """ Route key error """
+
+
+class NavigationItem:
+    """ Navigation item """
+
+    def __init__(self, routeKey: str, parentRouteKey: str, widget: NavigationWidget):
+        self.routeKey = routeKey
+        self.parentRouteKey = parentRouteKey
+        self.widget = widget
+
+
 class NavigationPanel(QFrame):
     """ Navigation panel """
 
@@ -38,8 +63,9 @@ class NavigationPanel(QFrame):
         self._parent = parent   # type: QWidget
         self._isMenuButtonVisible = True
         self._isReturnButtonVisible = False
+        self._isCollapsible = True
 
-        self.scrollArea = ScrollArea(self)
+        self.scrollArea = SingleDirectionScrollArea(self)
         self.scrollWidget = QWidget()
 
         self.menuButton = NavigationToolButton(FIF.MENU, self)
@@ -50,8 +76,8 @@ class NavigationPanel(QFrame):
         self.bottomLayout = NavigationItemLayout()
         self.scrollLayout = NavigationItemLayout(self.scrollWidget)
 
-        self.items = {}   # type: Dict[str, NavigationWidget]
-        self.history = NavigationHistory(self.items)
+        self.items = {}   # type: Dict[str, NavigationItem]
+        self.history = qrouter
 
         self.expandAni = QPropertyAnimation(self, b'geometry', self)
         self.expandWidth = 322
@@ -85,9 +111,17 @@ class NavigationPanel(QFrame):
         self.history.emptyChanged.connect(self.returnButton.setDisabled)
         self.returnButton.clicked.connect(self.history.pop)
 
+        # add tool tip
+        self.returnButton.installEventFilter(ToolTipFilter(self.returnButton, 1000))
+        self.returnButton.setToolTip(self.tr('Back'))
+
+        self.menuButton.installEventFilter(ToolTipFilter(self.menuButton, 1000))
+        self.menuButton.setToolTip(self.tr('Open Navigation'))
+
         self.setProperty('menu', False)
         self.scrollWidget.setObjectName('scrollWidget')
         FluentStyleSheet.NAVIGATION_INTERFACE.apply(self)
+        FluentStyleSheet.NAVIGATION_INTERFACE.apply(self.scrollWidget)
         self.__initLayout()
 
     def __initLayout(self):
@@ -112,7 +146,14 @@ class NavigationPanel(QFrame):
         self.topLayout.addWidget(self.returnButton, 0, Qt.AlignmentFlag.AlignTop)
         self.topLayout.addWidget(self.menuButton, 0, Qt.AlignmentFlag.AlignTop)
 
-    def addItem(self, routeKey: str, icon: Union[str, QIcon, FluentIconBase], text: str, onClick, selectable=True, position=NavigationItemPostion.TOP):
+    def widget(self, routeKey: str):
+        if routeKey not in self.items:
+            raise RouteKeyError(f"`{routeKey}` is illegal.")
+
+        return self.items[routeKey].widget
+
+    def addItem(self, routeKey: str, icon: Union[str, QIcon, FluentIconBase], text: str, onClick=None, selectable=True,
+                position=NavigationItemPosition.TOP, tooltip: str = None, parentRouteKey: str = None):
         """ add navigation item
 
         Parameters
@@ -129,19 +170,22 @@ class NavigationPanel(QFrame):
         onClick: callable
             the slot connected to item clicked signal
 
-        position: NavigationItemPostion
+        position: NavigationItemPosition
             where the button is added
 
         selectable: bool
             whether the item is selectable
+
+        tooltip: str
+            the tooltip of item
+
+        parentRouteKey: str
+            the route key of parent item, the parent widget should be `NavigationTreeWidget`
         """
-        if routeKey in self.items:
-            return
+        return self.insertItem(-1, routeKey, icon, text, onClick, selectable, position, tooltip, parentRouteKey)
 
-        button = NavigationPushButton(icon, text, selectable, self)
-        self.addWidget(routeKey, button, onClick, position)
-
-    def addWidget(self, routeKey: str, widget: NavigationWidget, onClick, position=NavigationItemPostion.TOP):
+    def addWidget(self, routeKey: str, widget: NavigationWidget, onClick=None, position=NavigationItemPosition.TOP,
+                  tooltip: str = None, parentRouteKey: str = None):
         """ add custom widget
 
         Parameters
@@ -155,20 +199,94 @@ class NavigationPanel(QFrame):
         onClick: callable
             the slot connected to item clicked signal
 
-        position: NavigationItemPostion
+        position: NavigationItemPosition
             where the button is added
+
+        tooltip: str
+            the tooltip of widget
+
+        parentRouteKey: str
+            the route key of parent item, the parent item should be `NavigationTreeWidget`
+        """
+        self.insertWidget(-1, routeKey, widget, onClick, position, tooltip, parentRouteKey)
+
+    def insertItem(self, index: int, routeKey: str, icon: Union[str, QIcon, FluentIconBase], text: str, onClick=None,
+                   selectable=True, position=NavigationItemPosition.TOP, tooltip: str = None, parentRouteKey=None):
+        """ insert navigation tree item
+
+        Parameters
+        ----------
+        index: int
+            the insert position of parent widget
+
+        routeKey: str
+            the unique name of item
+
+        icon: str | QIcon | FluentIconBase
+            the icon of navigation item
+
+        text: str
+            the text of navigation item
+
+        onClick: callable
+            the slot connected to item clicked signal
+
+        position: NavigationItemPosition
+            where the button is added
+
+        selectable: bool
+            whether the item is selectable
+
+        tooltip: str
+            the tooltip of item
+
+        parentRouteKey: str
+            the route key of parent item, the parent item should be `NavigationTreeWidget`
         """
         if routeKey in self.items:
             return
 
-        widget.clicked.connect(self._onWidgetClicked)
-        widget.clicked.connect(onClick)
-        widget.setProperty('routeKey', routeKey)
-        self.items[routeKey] = widget
+        w = NavigationTreeWidget(icon, text, selectable, self)
+        self.insertWidget(index, routeKey, w, onClick, position, tooltip, parentRouteKey)
+        return w
 
-        self._addWidgetToLayout(widget, position)
+    def insertWidget(self, index: int, routeKey: str, widget: NavigationWidget, onClick=None,
+                     position=NavigationItemPosition.TOP, tooltip: str = None, parentRouteKey: str = None):
+        """ insert custom widget
 
-    def addSeparator(self, position=NavigationItemPostion.TOP):
+        Parameters
+        ----------
+        index: int
+            insert position
+
+        routeKey: str
+            the unique name of item
+
+        widget: NavigationWidget
+            the custom widget to be added
+
+        onClick: callable
+            the slot connected to item clicked signal
+
+        position: NavigationItemPosition
+            where the button is added
+
+        tooltip: str
+            the tooltip of widget
+
+        parentRouteKey: str
+            the route key of parent item, the parent item should be `NavigationTreeWidget`
+        """
+        if routeKey in self.items:
+            return
+
+        self._registerWidget(routeKey, parentRouteKey, widget, onClick, tooltip)
+        if parentRouteKey:
+            self.widget(parentRouteKey).insertChild(index, widget)
+        else:
+            self._insertWidgetToLayout(index, widget, position)
+
+    def addSeparator(self, position=NavigationItemPosition.TOP):
         """ add separator
 
         Parameters
@@ -176,33 +294,98 @@ class NavigationPanel(QFrame):
         position: NavigationPostion
             where to add the separator
         """
-        separator = NavigationSeparator(self)
-        self._addWidgetToLayout(separator, position)
+        self.insertSeparator(-1, position)
 
-    def _addWidgetToLayout(self, widget: NavigationWidget, position: NavigationItemPostion):
-        """ add widget to layout """
-        if position == NavigationItemPostion.TOP:
+    def insertSeparator(self, index: int, position=NavigationItemPosition.TOP):
+        """ add separator
+
+        Parameters
+        ----------
+        index: int
+            insert position
+
+        position: NavigationPostion
+            where to add the separator
+        """
+        separator = NavigationSeparator(self)
+        self._insertWidgetToLayout(index, separator, position)
+
+    def _registerWidget(self, routeKey: str, parentRouteKey: str, widget: NavigationWidget, onClick, tooltip: str):
+        """ register widget """
+        widget.clicked.connect(self._onWidgetClicked)
+
+        if onClick is not None:
+            widget.clicked.connect(onClick)
+
+        widget.setProperty('routeKey', routeKey)
+        widget.setProperty('parentRouteKey', parentRouteKey)
+        self.items[routeKey] = NavigationItem(routeKey, parentRouteKey, widget)
+
+        if self.displayMode in [NavigationDisplayMode.EXPAND, NavigationDisplayMode.MENU]:
+            widget.setCompacted(False)
+
+        if tooltip:
+            widget.setToolTip(tooltip)
+            widget.installEventFilter(NavigationToolTipFilter(widget, 1000))
+
+    def _insertWidgetToLayout(self, index: int, widget: NavigationWidget, position: NavigationItemPosition):
+        """ insert widget to layout """
+        if position == NavigationItemPosition.TOP:
             widget.setParent(self)
-            self.topLayout.addWidget(widget, 0, Qt.AlignmentFlag.AlignTop)
-        elif position == NavigationItemPostion.SCROLL:
+            self.topLayout.insertWidget(index, widget, 0, Qt.AlignmentFlag.AlignTop)
+        elif position == NavigationItemPosition.SCROLL:
             widget.setParent(self.scrollWidget)
-            self.scrollLayout.addWidget(widget, 0, Qt.AlignmentFlag.AlignTop)
+            self.scrollLayout.insertWidget(index, widget, 0, Qt.AlignmentFlag.AlignTop)
         else:
             widget.setParent(self)
-            self.bottomLayout.addWidget(widget, 0, Qt.AlignmentFlag.AlignBottom)
+            self.bottomLayout.insertWidget(index, widget, 0, Qt.AlignmentFlag.AlignBottom)
 
         widget.show()
 
+    def removeWidget(self, routeKey: str):
+        """ remove widget
+
+        Parameters
+        ----------
+        routeKey: str
+            the unique name of item
+        """
+        if routeKey not in self.items:
+            return
+
+        item = self.items.pop(routeKey)
+
+        if item.parentRouteKey is not None:
+            self.widget(item.parentRouteKey).removeChild(item.widget)
+
+        if isinstance(item.widget, NavigationTreeWidgetBase):
+            for child in item.widget.findChildren(NavigationWidget, options=Qt.FindChildrenRecursively):
+                key = child.property('routeKey')
+                if key is None:
+                    continue
+
+                self.items.pop(key)
+                child.deleteLater()
+                self.history.remove(key)
+
+        item.widget.deleteLater()
+        self.history.remove(routeKey)
+
     def setMenuButtonVisible(self, isVisible: bool):
         """ set whether the menu button is visible """
         self._isMenuButtonVisible = isVisible
         self.menuButton.setVisible(isVisible)
 
     def setReturnButtonVisible(self, isVisible: bool):
-        """ set whether the menu button is visible """
+        """ set whether the return button is visible """
         self._isReturnButtonVisible = isVisible
         self.returnButton.setVisible(isVisible)
 
+    def setCollapsible(self, on: bool):
+        self._isCollapsible = on
+        if not on and self.displayMode != NavigationDisplayMode.EXPAND:
+            self.expand(False)
+
     def setExpandWidth(self, width: int):
         """ set the maximum width """
         if width <= 42:
@@ -211,15 +394,16 @@ class NavigationPanel(QFrame):
         self.expandWidth = width
         NavigationWidget.EXPAND_WIDTH = width - 10
 
-    def expand(self):
+    def expand(self, useAni=True):
         """ expand navigation panel """
         self._setWidgetCompacted(False)
         self.expandAni.setProperty('expand', True)
+        self.menuButton.setToolTip(self.tr('Close Navigation'))
 
         # determine the display mode according to the width of window
         # https://learn.microsoft.com/en-us/windows/apps/design/controls/navigationview#default
         expandWidth = 1007 + self.expandWidth - 322
-        if self.window().width() > expandWidth and not self.isMinimalEnabled:
+        if (self.window().width() > expandWidth and not self.isMinimalEnabled) or not self._isCollapsible:
             self.displayMode = NavigationDisplayMode.EXPAND
         else:
             self.setProperty('menu', True)
@@ -233,18 +417,27 @@ class NavigationPanel(QFrame):
 
             self.show()
 
-        self.displayModeChanged.emit(self.displayMode)
-        self.expandAni.setStartValue(
-            QRect(self.pos(), QSize(48, self.height())))
-        self.expandAni.setEndValue(
-            QRect(self.pos(), QSize(self.expandWidth, self.height())))
-        self.expandAni.start()
+        if useAni:
+            self.displayModeChanged.emit(self.displayMode)
+            self.expandAni.setStartValue(
+                QRect(self.pos(), QSize(48, self.height())))
+            self.expandAni.setEndValue(
+                QRect(self.pos(), QSize(self.expandWidth, self.height())))
+            self.expandAni.start()
+        else:
+            self.setFixedWidth(self.expandWidth)
+            self._onExpandAniFinished()
 
     def collapse(self):
         """ collapse navigation panel """
         if self.expandAni.state() == QPropertyAnimation.State.Running:
             return
 
+        for item in self.items.values():
+            w = item.widget
+            if isinstance(w, NavigationTreeWidgetBase) and w.isRoot():
+                w.setExpanded(False)
+
         self.expandAni.setStartValue(
             QRect(self.pos(), QSize(self.width(), self.height())))
         self.expandAni.setEndValue(
@@ -252,6 +445,8 @@ class NavigationPanel(QFrame):
         self.expandAni.setProperty('expand', False)
         self.expandAni.start()
 
+        self.menuButton.setToolTip(self.tr('Open Navigation'))
+
     def toggle(self):
         """ toggle navigation panel """
         if self.displayMode in [NavigationDisplayMode.COMPACT, NavigationDisplayMode.MINIMAL]:
@@ -270,9 +465,8 @@ class NavigationPanel(QFrame):
         if routeKey not in self.items:
             return
 
-        self.history.push(routeKey)
         for k, item in self.items.items():
-            item.setSelected(k == routeKey)
+            item.widget.setSelected(k == routeKey)
 
     def _onWidgetClicked(self):
         widget = self.sender()  # type: NavigationWidget
@@ -280,7 +474,8 @@ class NavigationPanel(QFrame):
             return
 
         self.setCurrentItem(widget.property('routeKey'))
-        if widget is not self.menuButton and self.displayMode == NavigationDisplayMode.MENU:
+        if widget is not self.menuButton and self.displayMode == NavigationDisplayMode.MENU \
+                and not (isinstance(widget, NavigationTreeWidgetBase) and not widget.isLeaf()):
             self.collapse()
 
     def resizeEvent(self, e: QResizeEvent):
@@ -293,7 +488,7 @@ class NavigationPanel(QFrame):
         self.scrollArea.setFixedHeight(max(h, 36))
 
     def eventFilter(self, obj, e: QEvent):
-        if obj is not self.window():
+        if obj is not self.window() or not self._isCollapsible:
             return super().eventFilter(obj, e)
 
         if e.type() == QEvent.Type.MouseButtonRelease:
@@ -327,7 +522,7 @@ class NavigationPanel(QFrame):
             self.setStyle(QApplication.style())
 
             for item in self.items.values():
-                item.setCompacted(True)
+                item.widget.setCompacted(True)
 
             if not self._parent.isWindow():
                 self.setParent(self._parent)
@@ -347,9 +542,6 @@ class NavigationPanel(QFrame):
         spacing += self.bottomLayout.count() * self.bottomLayout.spacing()
         return 36 + th + bh + sh + spacing
 
-    def setDefaultRouteKey(self, key: str):
-        """ set the routing key to use when the navigation history is empty """
-        self.history.defaultRouteKey = key
 
 class NavigationItemLayout(QVBoxLayout):
     """ Navigation layout """
@@ -362,46 +554,3 @@ class NavigationItemLayout(QVBoxLayout):
                 geo = item.geometry()
                 item.widget().setGeometry(0, geo.y(), geo.width(), geo.height())
 
-
-class NavigationHistory(QObject):
-    """ Navigation history """
-
-    emptyChanged = pyqtSignal(bool)
-
-    def __init__(self, items: Dict[str, NavigationWidget]):
-        super().__init__()
-        self.items = items
-        self.history = []
-        self._defaultRouteKey = None
-
-    @property
-    def defaultRouteKey(self):
-        return self._defaultRouteKey
-
-    @defaultRouteKey.setter
-    def defaultRouteKey(self, key):
-        if key not in self.items:
-            raise ValueError(f'The route key `{key}` has not been registered yet.')
-
-        self._defaultRouteKey = key
-
-    def push(self, routeKey: str):
-        if not self.history and self.defaultRouteKey != routeKey:
-            self.history.append(routeKey)
-            self.emptyChanged.emit(False)
-        elif self.history and self.history[-1] != routeKey:
-            self.history.append(routeKey)
-
-    def pop(self):
-        if not self.history:
-            return
-
-        self.history.pop()
-
-        if self.history:
-            self.items[self.history[-1]].clicked.emit(False)
-        else:
-            if self.defaultRouteKey is not None:
-                self.items[self.defaultRouteKey].clicked.emit(False)
-
-            self.emptyChanged.emit(True)
