@@ -137,6 +137,9 @@ class SwitchSettingCard(SettingCard):
         self.switchButton.setText(
             self.tr('On') if isChecked else self.tr('Off'))
 
+    def setChecked(self, isChecked: bool):
+        self.setValue(isChecked)
+
     def isChecked(self):
         return self.switchButton.isChecked()
 
@@ -195,6 +198,7 @@ class RangeSettingCard(SettingCard):
         qconfig.set(self.configItem, value)
         self.valueLabel.setNum(value)
         self.valueLabel.adjustSize()
+        self.slider.setValue(value)
 
 
 class PushSettingCard(SettingCard):
@@ -275,9 +279,10 @@ class ColorPickerButton(QToolButton):
 
     colorChanged = pyqtSignal(QColor)
 
-    def __init__(self, color: QColor, title: str, parent=None):
+    def __init__(self, color: QColor, title: str, parent=None, enableAlpha=False):
         super().__init__(parent=parent)
         self.title = title
+        self.enableAlpha = enableAlpha
         self.setFixedSize(96, 32)
         self.setAttribute(Qt.WidgetAttribute.WA_TranslucentBackground)
 
@@ -288,8 +293,7 @@ class ColorPickerButton(QToolButton):
     def __showColorDialog(self):
         """ show color dialog """
         w = ColorDialog(self.color, self.tr(
-            'Choose ')+self.title, self.window())
-        w.updateStyle()
+            'Choose ')+self.title, self.window(), self.enableAlpha)
         w.colorChanged.connect(self.__onColorChanged)
         w.exec()
 
@@ -308,7 +312,12 @@ class ColorPickerButton(QToolButton):
         painter.setRenderHints(QPainter.RenderHint.Antialiasing)
         pc = QColor(255, 255, 255, 10) if isDarkTheme() else QColor(234, 234, 234)
         painter.setPen(pc)
-        painter.setBrush(self.color)
+
+        color = QColor(self.color)
+        if not self.enableAlpha:
+            color.setAlpha(255)
+
+        painter.setBrush(color)
         painter.drawRoundedRect(self.rect().adjusted(1, 1, -1, -1), 5, 5)
 
 
@@ -317,7 +326,8 @@ class ColorSettingCard(SettingCard):
 
     colorChanged = pyqtSignal(QColor)
 
-    def __init__(self, configItem, icon: Union[str, QIcon, FluentIconBase], title, content=None, parent=None):
+    def __init__(self, configItem, icon: Union[str, QIcon, FluentIconBase],
+                 title: str, content: str = None, parent=None, enableAlpha=False):
         """
         Parameters
         ----------
@@ -335,11 +345,14 @@ class ColorSettingCard(SettingCard):
 
         parent: QWidget
             parent widget
+
+        enableAlpha: bool
+            whether to enable the alpha channel
         """
         super().__init__(icon, title, content, parent)
         self.configItem = configItem
         self.colorPicker = ColorPickerButton(
-            qconfig.get(configItem), title, self)
+            qconfig.get(configItem), title, self, enableAlpha)
         self.hBoxLayout.addWidget(self.colorPicker, 0, Qt.AlignmentFlag.AlignRight)
         self.hBoxLayout.addSpacing(16)
         self.colorPicker.colorChanged.connect(self.__onColorChanged)
