@@ -1,15 +1,17 @@
 # coding:utf-8
-from typing import Dict, Union, List, Iterable
+from typing import Union, List, Iterable
 
-from PyQt6.QtCore import Qt, pyqtSignal, QRect, QRectF, QPoint, QObject, QEvent
-from PyQt6.QtGui import QColor, QPainter, QAction, QCursor, QIcon
-from PyQt6.QtWidgets import QPushButton, QWidget, QStyledItemDelegate, QStyle
+from PyQt6.QtCore import Qt, pyqtSignal, QRectF, QPoint, QObject, QEvent
+from PyQt6.QtGui import QPainter, QAction, QCursor, QIcon
+from PyQt6.QtWidgets import QPushButton, QStyledItemDelegate, QStyle
 
-from .menu import RoundMenu
+from .menu import RoundMenu, MenuAnimationType, IndicatorMenuItemDelegate
 from .line_edit import LineEdit, LineEditButton
+from ...common.animation import TranslateYAnimation
 from ...common.icon import FluentIconBase, isDarkTheme
 from ...common.icon import FluentIcon as FIF
-from ...common.style_sheet import FluentStyleSheet, themeColor
+from ...common.font import setFont
+from ...common.style_sheet import FluentStyleSheet
 
 
 class ComboItem:
@@ -59,8 +61,8 @@ class ComboBoxBase(QObject):
         self.isHover = False
         self.isPressed = False
         self.items = []     # type: List[ComboItem]
-        self.itemMap = {}   # type: Dict[str, ComboItem]
         self._currentIndex = -1
+        self._maxVisibleItems = -1
         self.dropMenu = None
 
         FluentStyleSheet.COMBO_BOX.apply(self)
@@ -89,12 +91,10 @@ class ComboBoxBase(QObject):
 
         icon: str | QIcon | FluentIconBase
         """
-        if not text or text in self.itemMap:
-            return
-
         item = ComboItem(text, icon, userData)
-        self.itemMap[text] = item
         self.items.append(item)
+        if len(self.items) == 1:
+            self.setCurrentIndex(0)
 
     def addItems(self, texts: Iterable[str]):
         """ add items
@@ -114,9 +114,7 @@ class ComboBoxBase(QObject):
         if not 0 <= index < len(self.items):
             return
 
-        item = self.items[index]
         self.items.pop(index)
-        self.itemMap.pop(item.text)
 
         if index < self.currentIndex():
             self._onItemClicked(self._currentIndex - 1)
@@ -170,12 +168,14 @@ class ComboBoxBase(QObject):
         text: str
             text displayed in combo box
         """
-        if text not in self.itemMap or text == self.currentText():
+        if text == self.currentText():
             return
 
-        self.setCurrentIndex(self.items.index(self.itemMap[text]))
+        index = self.findText(text)
+        if index >= 0:
+            self.setCurrentIndex(index)
 
-    def setItemText(self, index, text):
+    def setItemText(self, index: int, text: str):
         """ set the text of item
 
         Parameters
@@ -186,12 +186,10 @@ class ComboBoxBase(QObject):
         text: str
             new text of item
         """
-        if text in self.itemMap or not 0 <= index < len(self.items):
+        if not 0 <= index < len(self.items):
             return
 
-        item = self.itemMap.pop()
-        item.text = text
-        self.itemMap[text] = item
+        self.items[index].text = text
         if self.currentIndex() == index:
             self.setText(text)
 
@@ -236,10 +234,11 @@ class ComboBoxBase(QObject):
 
     def findText(self, text: str):
         """ Returns the index of the item containing the given text; otherwise returns -1. """
-        if text not in self.itemMap:
-            return -1
+        for i, item in enumerate(self.items):
+            if item.text == text:
+                return i
 
-        return self.items.index(self.itemMap[text])
+        return -1
 
     def clear(self):
         """ Clears the combobox, removing all items. """
@@ -247,7 +246,6 @@ class ComboBoxBase(QObject):
             self.setText('')
 
         self.items.clear()
-        self.itemMap.clear()
         self._currentIndex = -1
 
     def count(self):
@@ -256,12 +254,8 @@ class ComboBoxBase(QObject):
 
     def insertItem(self, index: int, text: str, icon: Union[str, QIcon, FluentIconBase] = None, userData=None):
         """ Inserts item into the combobox at the given index. """
-        if not text or text in self.itemMap:
-            return
-
         item = ComboItem(text, icon, userData)
         self.items.insert(index, item)
-        self.itemMap[text] = item
 
         if index <= self.currentIndex():
             self._onItemClicked(self.currentIndex() + 1)
@@ -270,17 +264,19 @@ class ComboBoxBase(QObject):
         """ Inserts items into the combobox, starting at the index specified. """
         pos = index
         for text in texts:
-            if not text or text in self.itemMap:
-                continue
-
             item = ComboItem(text)
             self.items.insert(pos, item)
-            self.itemMap[text] = item
             pos += 1
 
         if index <= self.currentIndex():
             self._onItemClicked(self.currentIndex() + pos - index)
 
+    def setMaxVisibleItems(self, num: int):
+        self._maxVisibleItems = num
+
+    def maxVisibleItems(self):
+        return self._maxVisibleItems
+
     def _closeComboMenu(self):
         if not self.dropMenu:
             return
@@ -306,6 +302,7 @@ class ComboBoxBase(QObject):
             menu.view.setMinimumWidth(self.width())
             menu.adjustSize()
 
+        menu.setMaxVisibleItems(self.maxVisibleItems())
         menu.closedSignal.connect(self._onDropMenuClosed)
         self.dropMenu = menu
 
@@ -313,10 +310,20 @@ class ComboBoxBase(QObject):
         if self.currentIndex() >= 0 and self.items:
             menu.setDefaultAction(menu.menuActions()[self.currentIndex()])
 
-        # show menu
+        # determine the animation type by choosing the maximum height of view
         x = -menu.width()//2 + menu.layout().contentsMargins().left() + self.width()//2
-        y = self.height()
-        menu.exec(self.mapToGlobal(QPoint(x, y)))
+        pd = self.mapToGlobal(QPoint(x, self.height()))
+        hd = menu.view.heightForAnimation(pd, MenuAnimationType.DROP_DOWN)
+
+        pu = self.mapToGlobal(QPoint(x, 0))
+        hu = menu.view.heightForAnimation(pd, MenuAnimationType.PULL_UP)
+
+        if hd >= hu:
+            menu.view.adjustSize(pd, MenuAnimationType.DROP_DOWN)
+            menu.exec(pd, aniType=MenuAnimationType.DROP_DOWN)
+        else:
+            menu.view.adjustSize(pu, MenuAnimationType.PULL_UP)
+            menu.exec(pu, aniType=MenuAnimationType.PULL_UP)
 
     def _toggleComboMenu(self):
         if self.dropMenu:
@@ -341,6 +348,8 @@ class ComboBox(QPushButton, ComboBoxBase):
 
     def __init__(self, parent=None):
         super().__init__(parent=parent)
+        self.arrowAni = TranslateYAnimation(self)
+        setFont(self)
 
     def setPlaceholderText(self, text: str):
         self.setText(text)
@@ -358,7 +367,7 @@ class ComboBox(QPushButton, ComboBoxBase):
         elif self.isPressed:
             painter.setOpacity(0.7)
 
-        rect = QRectF(self.width()-22, self.height()/2-5, 10, 10)
+        rect = QRectF(self.width()-22, self.height()/2-5+self.arrowAni.y, 10, 10)
         if isDarkTheme():
             FIF.ARROW_DOWN.render(painter, rect)
         else:
@@ -381,40 +390,45 @@ class EditableComboBox(LineEdit, ComboBoxBase):
 
         self.dropButton.clicked.connect(self._toggleComboMenu)
         self.textEdited.connect(self._onTextEdited)
-        self.returnPressed.connect(lambda: self.addItem(self.text()))
+        self.returnPressed.connect(self._onReturnPressed)
+
+        self.clearButton.disconnect()
+        self.clearButton.clicked.connect(self._onClearButtonClicked)
+
+    def currentText(self):
+        return self.text()
+
+    def clear(self):
+        ComboBoxBase.clear(self)
+
+    def _onReturnPressed(self):
+        if not self.text():
+            return
+
+        index = self.findText(self.text())
+        if index >= 0 and index != self.currentIndex():
+            self._currentIndex = index
+            self.currentIndexChanged.emit(index)
+        elif index == -1:
+            self.addItem(self.text())
+            self.setCurrentIndex(self.count() - 1)
 
     def _onTextEdited(self, text: str):
-        if text not in self.itemMap:
-            self._currentIndex = -1
-        else:
-            self._currentIndex = self.items.index(self.itemMap[text])
-
+        self._currentIndex = -1
         self.currentTextChanged.emit(text)
 
+        for i, item in enumerate(self.items):
+            if item.text == text:
+                self._currentIndex = i
+                self.currentIndexChanged.emit(i)
+                return
+
     def _onDropMenuClosed(self):
         self.dropMenu = None
 
-
-class ComboMenuItemDelegate(QStyledItemDelegate):
-    """ Combo box drop menu item delegate """
-
-    def paint(self, painter: QPainter, option, index):
-        super().paint(painter, option, index)
-        if not option.state & QStyle.StateFlag.State_Selected:
-            return
-
-        painter.save()
-        painter.setRenderHints(
-            QPainter.RenderHint.Antialiasing |
-            QPainter.RenderHint.SmoothPixmapTransform |
-            QPainter.RenderHint.TextAntialiasing
-        )
-
-        painter.setPen(Qt.PenStyle.NoPen)
-        painter.setBrush(themeColor())
-        painter.drawRoundedRect(0, 11+option.rect.y(), 3, 15, 1.5, 1.5)
-
-        painter.restore()
+    def _onClearButtonClicked(self):
+        LineEdit.clear(self)
+        self._currentIndex = -1
 
 
 class ComboBoxMenu(RoundMenu):
@@ -423,8 +437,14 @@ class ComboBoxMenu(RoundMenu):
     def __init__(self, parent=None):
         super().__init__(title="", parent=parent)
 
-        self.view.setViewportMargins(5, 2, 5, 6)
-        self.view.setItemDelegate(ComboMenuItemDelegate())
+        self.view.setViewportMargins(0, 2, 0, 6)
+        self.view.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAsNeeded)
+        self.view.setItemDelegate(IndicatorMenuItemDelegate())
+        self.view.setObjectName('comboListWidget')
 
-        FluentStyleSheet.COMBO_BOX.apply(self)
         self.setItemHeight(33)
+
+    def exec(self, pos, ani=True, aniType=MenuAnimationType.DROP_DOWN):
+        self.view.adjustSize(pos, aniType)
+        self.adjustSize()
+        return super().exec(pos, ani, aniType)
\ No newline at end of file
