@@ -13,26 +13,9 @@ from ...common.auto_wrap import TextWrap
 from ...common.style_sheet import FluentStyleSheet, themeColor
 from ...common.icon import FluentIconBase, Theme, isDarkTheme, writeSvg, drawSvgIcon, drawIcon
 from ...common.icon import FluentIcon as FIF
+from .button import TransparentToolButton
 
 
-class InfoBarCloseButton(QToolButton):
-    """ Close button """
-
-    def __init__(self, parent=None):
-        super().__init__(parent=parent)
-        self.setFixedSize(36, 36)
-        self.setIconSize(QSize(12, 12))
-        self.setCursor(Qt.CursorShape.PointingHandCursor)
-        self.setObjectName('infoBarCloseButton')
-        FluentStyleSheet.INFO_BAR.apply(self)
-
-    def paintEvent(self, e):
-        super().paintEvent(e)
-        painter = QPainter(self)
-        painter.setRenderHints(QPainter.RenderHint.Antialiasing |
-                               QPainter.RenderHint.SmoothPixmapTransform)
-        FIF.CLOSE.render(painter, QRectF(12, 12, 12, 12))
-
 
 class InfoBarIcon(FluentIconBase, Enum):
     """ Info bar icon """
@@ -67,7 +50,7 @@ class InfoIconWidget(QWidget):
 
     def __init__(self, icon: InfoBarIcon, parent=None):
         super().__init__(parent=parent)
-        self.setFixedSize(15, 15)
+        self.setFixedSize(36, 36)
         self.icon = icon
 
     def paintEvent(self, e):
@@ -75,10 +58,11 @@ class InfoIconWidget(QWidget):
         painter.setRenderHints(QPainter.RenderHint.Antialiasing |
                                QPainter.RenderHint.SmoothPixmapTransform)
 
+        rect = QRectF(10, 10, 15, 15)
         if self.icon != InfoBarIcon.INFORMATION:
-            drawIcon(self.icon, painter, self.rect())
+            drawIcon(self.icon, painter, rect)
         else:
-            drawIcon(self.icon, painter, self.rect(), indexes=[0], fill=themeColor().name())
+            drawIcon(self.icon, painter, rect, indexes=[0], fill=themeColor().name())
 
 
 class InfoBar(QFrame):
@@ -123,14 +107,14 @@ class InfoBar(QFrame):
         self.isClosable = isClosable
         self.position = position
 
-        self.titleLabel = QLabel(title, self)
+        self.titleLabel = QLabel(self)
         self.contentLabel = QLabel(self)
-        self.closeButton = InfoBarCloseButton(self)
+        self.closeButton = TransparentToolButton(FIF.CLOSE, self)
         self.iconWidget = InfoIconWidget(icon)
 
-        self.vBoxLayout = QVBoxLayout(self)
-        self.hBoxLayout = QHBoxLayout()
-        self.contentLayout = QHBoxLayout() if self.orient == Qt.Orientation.Horizontal else QVBoxLayout()
+        self.hBoxLayout = QHBoxLayout(self)
+        self.textLayout = QHBoxLayout() if self.orient == Qt.Orientation.Horizontal else QVBoxLayout()
+        self.widgetLayout = QHBoxLayout() if self.orient == Qt.Orientation.Horizontal else QVBoxLayout()
 
         self.opacityEffect = QGraphicsOpacityEffect(self)
         self.opacityAni = QPropertyAnimation(
@@ -142,57 +126,56 @@ class InfoBar(QFrame):
         self.__initWidget()
 
     def __initWidget(self):
-        self.titleLabel.setMinimumHeight(36)
         self.opacityEffect.setOpacity(1)
         self.setGraphicsEffect(self.opacityEffect)
+
+        self.closeButton.setFixedSize(36, 36)
+        self.closeButton.setIconSize(QSize(12, 12))
+        self.closeButton.setCursor(Qt.CursorShape.PointingHandCursor)
         self.closeButton.setVisible(self.isClosable)
 
-        self.__initLayout()
         self.__setQss()
+        self.__initLayout()
 
         self.closeButton.clicked.connect(self.close)
 
     def __initLayout(self):
-        self.vBoxLayout.setContentsMargins(0, 0, 0, 0)
-        self.vBoxLayout.setSizeConstraint(QVBoxLayout.SizeConstraint.SetMinimumSize)
-        self.contentLayout.setSizeConstraint(QHBoxLayout.SizeConstraint.SetMinimumSize)
+        self.hBoxLayout.setContentsMargins(6, 6, 6, 6)
+        self.hBoxLayout.setSizeConstraint(QVBoxLayout.SizeConstraint.SetMinimumSize)
+        self.textLayout.setSizeConstraint(QHBoxLayout.SizeConstraint.SetMinimumSize)
+        self.textLayout.setAlignment(Qt.AlignmentFlag.AlignTop)
+        self.textLayout.setContentsMargins(1, 8, 0, 8)
 
-        self.vBoxLayout.setSpacing(0)
         self.hBoxLayout.setSpacing(0)
+        self.textLayout.setSpacing(5)
 
-        self.vBoxLayout.addLayout(self.hBoxLayout)
-        self.hBoxLayout.addWidget(self.iconWidget)
+        # add icon to layout
+        self.hBoxLayout.addWidget(self.iconWidget, 0, Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)
 
-        if self.title:
-            self.hBoxLayout.addSpacing(15)
-            self.hBoxLayout.addWidget(self.titleLabel)
-        else:
-            self.titleLabel.hide()
+        # add title to layout
+        self.textLayout.addWidget(self.titleLabel, 1, Qt.AlignmentFlag.AlignTop)
+        self.titleLabel.setVisible(bool(self.title))
 
         # add content label to layout
         if self.orient == Qt.Orientation.Horizontal:
-            self.vBoxLayout.setAlignment(Qt.AlignmentFlag.AlignVCenter)
-            self.hBoxLayout.addSpacing(12)
-            self.contentLayout.setContentsMargins(0, 0, 20*self.isClosable, 0)
-            self.contentLayout.setSpacing(12)
-            self.hBoxLayout.addLayout(self.contentLayout)
-        else:
-            self.vBoxLayout.setAlignment(Qt.AlignmentFlag.AlignTop)
-            self.contentLayout.setContentsMargins(47, 0, 40, 18)
-            self.contentLayout.setSpacing(14)
-            self.vBoxLayout.addLayout(self.contentLayout)
-            self.contentLayout.setAlignment(Qt.AlignmentFlag.AlignTop)
+            self.textLayout.addSpacing(7)
 
-        self._adjustText()
-        self.contentLayout.addWidget(self.contentLabel)
+        self.textLayout.addWidget(self.contentLabel, 1, Qt.AlignmentFlag.AlignTop)
+        self.contentLabel.setVisible(bool(self.content))
+        self.hBoxLayout.addLayout(self.textLayout)
+
+        # add widget layout
+        if self.orient == Qt.Orientation.Horizontal:
+            self.hBoxLayout.addLayout(self.widgetLayout)
+            self.widgetLayout.setSpacing(10)
+        else:
+            self.textLayout.addLayout(self.widgetLayout)
 
         # add close button to layout
-        if self.isClosable:
-            self.hBoxLayout.addWidget(self.closeButton)
-            mb = 6 if self.orient == Qt.Orientation.Horizontal else 0
-            self.hBoxLayout.setContentsMargins(16, 6, 7, mb)
-        else:
-            self.hBoxLayout.setContentsMargins(16, 6, 16, 6)
+        self.hBoxLayout.addSpacing(12)
+        self.hBoxLayout.addWidget(self.closeButton, 0, Qt.AlignmentFlag.AlignTop | Qt.AlignmentFlag.AlignLeft)
+
+        self._adjustText()
 
     def __setQss(self):
         self.titleLabel.setObjectName('titleLabel')
@@ -212,13 +195,21 @@ class InfoBar(QFrame):
 
     def _adjustText(self):
         w = 900 if not self.parent() else (self.parent().width() - 50)
+
+        # adjust title
+        chars = max(min(w / 10, 120), 30)
+        self.titleLabel.setText(TextWrap.wrap(self.title, chars, False)[0])
+
+        # adjust content
         chars = max(min(w / 9, 120), 30)
         self.contentLabel.setText(TextWrap.wrap(self.content, chars, False)[0])
         self.adjustSize()
 
     def addWidget(self, widget: QWidget, stretch=0):
         """ add widget to info bar """
-        self.contentLayout.addWidget(widget, stretch, Qt.AlignmentFlag.AlignLeft)
+        self.widgetLayout.addSpacing(6)
+        align = Qt.AlignmentFlag.AlignTop if self.orient == Qt.Orientation.Vertical else Qt.AlignmentFlag.AlignVCenter
+        self.widgetLayout.addWidget(widget, stretch, Qt.AlignmentFlag.AlignLeft | align)
 
     def setCustomBackgroundColor(self, light, dark):
         """ set the custom background color
@@ -307,6 +298,7 @@ class InfoBarManager(QObject):
     """ Info bar manager """
 
     _instance = None
+    managers = {}
 
     def __new__(cls, *args, **kwargs):
         if cls._instance is None:
@@ -409,9 +401,11 @@ class InfoBarManager(QObject):
             ani.setEndValue(self._pos(bar))
 
     def _pos(self, infoBar: InfoBar, parentSize=None) -> QPoint:
+        """ return the position of info bar """
         raise NotImplementedError
 
     def _slideStartPos(self, infoBar: InfoBar) -> QPoint:
+        """ return the start position of slide animation  """
         raise NotImplementedError
 
     def eventFilter(self, obj, e: QEvent):
@@ -425,24 +419,33 @@ class InfoBarManager(QObject):
 
         return super().eventFilter(obj, e)
 
-    @staticmethod
-    def make(position: InfoBarPosition):
+    @classmethod
+    def register(cls, name):
+        """ register menu animation manager
+
+        Parameters
+        ----------
+        name: Any
+            the name of manager, it should be unique
+        """
+        def wrapper(Manager):
+            if name not in cls.managers:
+                cls.managers[name] = Manager
+
+            return Manager
+
+        return wrapper
+
+    @classmethod
+    def make(cls, position: InfoBarPosition):
         """ mask info bar manager according to the display position """
-        managers = {
-            InfoBarPosition.TOP: TopInfoBarManager,
-            InfoBarPosition.BOTTOM: BottomInfoBarManager,
-            InfoBarPosition.TOP_RIGHT: TopRightInfoBarManager,
-            InfoBarPosition.BOTTOM_RIGHT: BottomRightInfoBarManager,
-            InfoBarPosition.TOP_LEFT: TopLeftInfoBarManager,
-            InfoBarPosition.BOTTOM_LEFT: BottomLeftInfoBarManager,
-        }
+        if position not in cls.managers:
+            raise ValueError(f'`{position}` is an invalid animation type.')
 
-        if position not in managers:
-            raise ValueError(f'`{position}` is an invalid info bar position.')
-
-        return managers[position]()
+        return cls.managers[position]()
 
 
+@InfoBarManager.register(InfoBarPosition.TOP)
 class TopInfoBarManager(InfoBarManager):
     """ Top position info bar manager """
 
@@ -463,6 +466,7 @@ class TopInfoBarManager(InfoBarManager):
         return QPoint(pos.x(), pos.y() - 16)
 
 
+@InfoBarManager.register(InfoBarPosition.TOP_RIGHT)
 class TopRightInfoBarManager(InfoBarManager):
     """ Top right position info bar manager """
 
@@ -482,6 +486,7 @@ class TopRightInfoBarManager(InfoBarManager):
         return QPoint(infoBar.parent().width(), self._pos(infoBar).y())
 
 
+@InfoBarManager.register(InfoBarPosition.BOTTOM_RIGHT)
 class BottomRightInfoBarManager(InfoBarManager):
     """ Bottom right position info bar manager """
 
@@ -502,6 +507,7 @@ class BottomRightInfoBarManager(InfoBarManager):
         return QPoint(infoBar.parent().width(), self._pos(infoBar).y())
 
 
+@InfoBarManager.register(InfoBarPosition.TOP_LEFT)
 class TopLeftInfoBarManager(InfoBarManager):
     """ Top left position info bar manager """
 
@@ -521,6 +527,7 @@ class TopLeftInfoBarManager(InfoBarManager):
         return QPoint(-infoBar.width(), self._pos(infoBar).y())
 
 
+@InfoBarManager.register(InfoBarPosition.BOTTOM_LEFT)
 class BottomLeftInfoBarManager(InfoBarManager):
     """ Bottom left position info bar manager """
 
@@ -540,6 +547,7 @@ class BottomLeftInfoBarManager(InfoBarManager):
         return QPoint(-infoBar.width(), self._pos(infoBar).y())
 
 
+@InfoBarManager.register(InfoBarPosition.BOTTOM)
 class BottomInfoBarManager(InfoBarManager):
     """ Bottom position info bar manager """
 
