@@ -1,17 +1,22 @@
 # coding:utf-8
+from enum import Enum
+from typing import List, Union
+
 from qframelesswindow import WindowEffect
-from PyQt6.QtCore import (QEasingCurve, QEvent, QPropertyAnimation, QRect,
-                          Qt, QSize, QRectF, pyqtSignal, QPoint, QTimer)
-from PyQt6.QtGui import QIcon, QAction, QColor, QPainter, QPen, QPixmap, QRegion, QCursor, QTextCursor
+from PyQt6.QtCore import (QEasingCurve, QEvent, QPropertyAnimation, QPointF, QModelIndex,
+                          Qt, QSize, QRectF, pyqtSignal, QPoint, QTimer, QObject, QParallelAnimationGroup)
+from PyQt6.QtGui import (QIcon, QAction, QColor, QPainter, QPen, QPixmap, QRegion, QCursor, QTextCursor, QHoverEvent,
+                         QFontMetrics, QKeySequence)
 from PyQt6.QtWidgets import (QApplication, QMenu, QProxyStyle, QStyle,
                              QGraphicsDropShadowEffect, QListWidget, QWidget, QHBoxLayout,
-                             QListWidgetItem, QLineEdit, QTextEdit)
+                             QListWidgetItem, QLineEdit, QTextEdit, QStyledItemDelegate, QStyleOptionViewItem)
 
-from ...common.smooth_scroll import SmoothScroll
 from ...common.icon import FluentIcon as FIF
-from ...common.icon import MenuIconEngine
-from ...common.style_sheet import FluentStyleSheet
+from ...common.icon import FluentIconEngine, Action, FluentIconBase, Icon
+from ...common.style_sheet import FluentStyleSheet, themeColor
+from ...common.font import getFont
 from ...common.config import isDarkTheme
+from .scroll_bar import SmoothScrollDelegate
 
 
 class CustomMenuStyle(QProxyStyle):
@@ -52,20 +57,15 @@ class DWMMenu(QMenu):
         return QMenu.event(self, e)
 
 
-class MenuSeparator(QWidget):
-    """ Menu separator """
+class MenuAnimationType(Enum):
+    """ Menu animation type """
 
-    def __init__(self, parent=None):
-        super().__init__(parent=parent)
-        self.setFixedHeight(9)
+    NONE = 0
+    DROP_DOWN = 1
+    PULL_UP = 2
+    FADE_IN_DROP_DOWN = 3
+    FADE_IN_PULL_UP = 4
 
-    def paintEvent(self, e):
-        painter = QPainter(self)
-        c = 0 if not isDarkTheme() else 255
-        pen = QPen(QColor(c, c, c, 25), 1)
-        pen.setCosmetic(True)
-        painter.setPen(pen)
-        painter.drawLine(0, 4, self.width(), 4)
 
 
 class SubMenuItemWidget(QWidget):
@@ -103,25 +103,85 @@ class SubMenuItemWidget(QWidget):
             self.width()-10, self.height()/2-9/2, 9, 9))
 
 
+class MenuItemDelegate(QStyledItemDelegate):
+    """ Menu item delegate """
+
+    def _isSeparator(self, index: QModelIndex):
+        return index.model().data(index, Qt.ItemDataRole.DecorationRole) == "seperator"
+
+    def paint(self, painter, option, index):
+        if not self._isSeparator(index):
+            return super().paint(painter, option, index)
+
+        # draw seperator
+        painter.save()
+
+        c = 0 if not isDarkTheme() else 255
+        pen = QPen(QColor(c, c, c, 25), 1)
+        pen.setCosmetic(True)
+        painter.setPen(pen)
+        rect = option.rect
+        painter.drawLine(0, rect.y() + 4, rect.width() + 12, rect.y() + 4)
+
+        painter.restore()
+
+
+class ShortcutMenuItemDelegate(MenuItemDelegate):
+    """ Shortcut key menu item delegate """
+
+    def paint(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
+        super().paint(painter, option, index)
+        if self._isSeparator(index):
+            return
+
+        # draw shortcut key
+        action = index.data(Qt.ItemDataRole.UserRole)  # type: QAction
+        if not isinstance(action, QAction) or action.shortcut().isEmpty():
+            return
+
+        painter.save()
+
+        if not option.state & QStyle.StateFlag.State_Enabled:
+            painter.setOpacity(0.5 if isDarkTheme() else 0.6)
+
+        font = getFont(12)
+        painter.setFont(font)
+        painter.setPen(QColor(255, 255, 255, 200) if isDarkTheme() else QColor(0, 0, 0, 153))
+
+        fm = QFontMetrics(font)
+        shortcut = action.shortcut().toString(QKeySequence.SequenceFormat.NativeText)
+
+        sw = fm.boundingRect(shortcut).width()
+        painter.translate(option.rect.width()-sw-20, 0)
+
+        rect = QRectF(0, option.rect.y(), sw, option.rect.height())
+        painter.drawText(rect, Qt.AlignmentFlag.AlignLeft | Qt.AlignmentFlag.AlignVCenter, shortcut)
+
+        painter.restore()
+
+
 class MenuActionListWidget(QListWidget):
     """ Menu action list widget """
 
     def __init__(self, parent=None):
         super().__init__(parent)
-        self.setViewportMargins(5, 6, 5, 6)
-        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
-        self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+        self._itemHeight = 28
+        self._maxVisibleItems = -1  # adjust visible items according to the size of screen
+
+        self.setViewportMargins(0, 6, 0, 6)
         self.setTextElideMode(Qt.TextElideMode.ElideNone)
         self.setDragEnabled(False)
         self.setMouseTracking(True)
         self.setVerticalScrollMode(self.ScrollMode.ScrollPerPixel)
         self.setIconSize(QSize(14, 14))
-        self.smoothScroll = SmoothScroll(self)
-        self.setStyleSheet(
-            'MenuActionListWidget{font: 14px "Segoe UI", "Microsoft YaHei"}')
+        self.setItemDelegate(ShortcutMenuItemDelegate(self))
 
-    def wheelEvent(self, e):
-        self.smoothScroll.wheelEvent(e)
+        self.scrollDelegate = SmoothScrollDelegate(self)
+        self.setStyleSheet(
+            'MenuActionListWidget{font: 14px "Segoe UI", "Microsoft YaHei", "PingFang SC"}')
+
+        self.setVerticalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
+        self.setHorizontalScrollBarPolicy(Qt.ScrollBarPolicy.ScrollBarAlwaysOff)
 
     def insertItem(self, row, item):
         """ inserts menu item at the position in the list given by row """
@@ -139,7 +199,7 @@ class MenuActionListWidget(QListWidget):
         self.adjustSize()
         return item
 
-    def adjustSize(self):
+    def adjustSize(self, pos=None, aniType=MenuAnimationType.NONE):
         size = QSize()
         for i in range(self.count()):
             s = self.item(i).sizeHint()
@@ -147,30 +207,57 @@ class MenuActionListWidget(QListWidget):
             size.setHeight(size.height() + s.height())
 
         # adjust the height of viewport
-        ss = QApplication.screenAt(QCursor.pos()).availableSize()
-        w, h = ss.width() - 100, ss.height() - 100
-        vsize = QSize(size)
-        vsize.setHeight(min(h-12, vsize.height()))
-        vsize.setWidth(min(w-12, vsize.width()))
+        w, h = MenuAnimationManager.make(self, aniType).availableViewSize(pos)
         self.viewport().adjustSize()
 
         # adjust the height of list widget
         m = self.viewportMargins()
         size += QSize(m.left()+m.right()+2, m.top()+m.bottom())
         size.setHeight(min(h, size.height()+3))
-        size.setWidth(min(w, size.width()))
+        size.setWidth(max(min(w, size.width()), self.minimumWidth()))
+
+        if self.maxVisibleItems() > 0:
+            size.setHeight(min(
+                size.height(), self.maxVisibleItems() * self._itemHeight + m.top()+m.bottom() + 3))
+
         self.setFixedSize(size)
 
-    def setItemHeight(self, height):
+    def setItemHeight(self, height: int):
         """ set the height of item """
+        if height == self._itemHeight:
+            return
+
         for i in range(self.count()):
             item = self.item(i)
-            item.setSizeHint(item.sizeHint().width(), height)
+            if not self.itemWidget(item):
+                item.setSizeHint(item.sizeHint().width(), height)
 
+        self._itemHeight = height
         self.adjustSize()
 
+    def setMaxVisibleItems(self, num: int):
+        """ set the maximum visible items """
+        self._maxVisibleItems = num
+        self.adjustSize()
 
-class RoundMenu(QWidget):
+    def maxVisibleItems(self):
+        return self._maxVisibleItems
+
+    def heightForAnimation(self, pos: QPoint, aniType: MenuAnimationType):
+        """ height for animation """
+        ih = self.itemsHeight()
+        _, sh = MenuAnimationManager.make(self, aniType).availableViewSize(pos)
+        return min(ih, sh)
+
+    def itemsHeight(self):
+        """ Return the height of all items """
+        N = self.count() if self.maxVisibleItems() < 0 else min(self.maxVisibleItems(), self.count())
+        h = sum(self.item(i).sizeHint().height() for i in range(N))
+        m = self.viewportMargins()
+        return h + m.top() + m.bottom()
+
+
+class RoundMenu(QMenu):
     """ Round corner menu """
 
     closedSignal = pyqtSignal()
@@ -179,8 +266,9 @@ class RoundMenu(QWidget):
         super().__init__(parent=parent)
         self._title = title
         self._icon = QIcon()
-        self._actions = []
+        self._actions = []  # type: List[QAction]
         self._subMenus = []
+
         self.isSubMenu = False
         self.parentMenu = None
         self.menuItem = None
@@ -188,10 +276,13 @@ class RoundMenu(QWidget):
         self.lastHoverSubMenuItem = None
         self.isHideBySystem = True
         self.itemHeight = 28
+
         self.hBoxLayout = QHBoxLayout(self)
         self.view = MenuActionListWidget(self)
-        self.ani = QPropertyAnimation(self, b'pos', self)
+
+        self.aniManager = None
         self.timer = QTimer(self)
+
         self.__initWidgets()
 
     def __initWidgets(self):
@@ -212,7 +303,11 @@ class RoundMenu(QWidget):
 
         self.view.itemClicked.connect(self._onItemClicked)
         self.view.itemEntered.connect(self._onItemEntered)
-        self.ani.valueChanged.connect(self._onSlideValueChanged)
+
+    def setMaxVisibleItems(self, num: int):
+        """ set the maximum visible items """
+        self.view.setMaxVisibleItems(num)
+        self.adjustSize()
 
     def setItemHeight(self, height):
         """ set the height of menu item """
@@ -253,11 +348,14 @@ class RoundMenu(QWidget):
         for i in range(len(self._actions)-1, -1, -1):
             self.removeAction(self._actions[i])
 
-    def setIcon(self, icon):
+    def setIcon(self, icon: Union[QIcon, FluentIconBase]):
         """ set the icon of menu """
+        if isinstance(icon, FluentIconBase):
+            icon = Icon(icon)
+
         self._icon = icon
 
-    def addAction(self, action):
+    def addAction(self, action: Union[QAction, Action]):
         """ add action to menu
 
         Parameters
@@ -269,7 +367,38 @@ class RoundMenu(QWidget):
         self.view.addItem(item)
         self.adjustSize()
 
-    def _createActionItem(self, action, before=None):
+    def addWidget(self, widget: QWidget, selectable=True, onClick=None):
+        """ add custom widget
+
+        Parameters
+        ----------
+        widget: QWidget
+            custom widget
+
+        selectable: bool
+            whether the menu item is selectable
+
+        onClick: callable
+            the slot connected to item clicked signal
+        """
+        action = QAction()
+        action.setProperty('selectable', selectable)
+
+        item = self._createActionItem(action)
+        item.setSizeHint(widget.size())
+
+        self.view.addItem(item)
+        self.view.setItemWidget(item, widget)
+
+        if not selectable:
+            item.setFlags(Qt.ItemFlag.NoItemFlags)
+
+        if onClick:
+            action.triggered.connect(onClick)
+
+        self.adjustSize()
+
+    def _createActionItem(self, action: QAction, before=None):
         """ create menu action item  """
         if not before:
             self._actions.append(action)
@@ -280,14 +409,12 @@ class RoundMenu(QWidget):
             raise ValueError('`before` is not in the action list')
 
         item = QListWidgetItem(self._createItemIcon(action), action.text())
-        if not self._hasItemIcon():
-            w = 28 + self.view.fontMetrics().boundingRect(action.text()).width()
-        else:
-            # add a blank character to increase space between icon and text
-            item.setText(" " + item.text())
-            w = 60 + self.view.fontMetrics().boundingRect(item.text()).width()
+        self._adjustItemText(item, action)
+
+        # disable item if the action is not enabled
+        if not action.isEnabled():
+            item.setFlags(Qt.ItemFlag.NoItemFlags)
 
-        item.setSizeHint(QSize(w, self.itemHeight))
         item.setData(Qt.ItemDataRole.UserRole, action)
         action.setProperty('item', item)
         action.changed.connect(self._onActionChanged)
@@ -296,10 +423,37 @@ class RoundMenu(QWidget):
     def _hasItemIcon(self):
         return any(not i.icon().isNull() for i in self._actions+self._subMenus)
 
+    def _adjustItemText(self, item: QListWidgetItem, action: QAction):
+        """ adjust the text of item """
+        # leave some space for shortcut key
+        if isinstance(self.view.itemDelegate(), ShortcutMenuItemDelegate):
+            sw = self._longestShortcutWidth()
+            if sw:
+                sw += 22
+        else:
+            sw = 0
+
+        # adjust the width of item
+        if not self._hasItemIcon():
+            item.setText(action.text())
+            w = 40 + self.view.fontMetrics().boundingRect(action.text()).width() + sw
+        else:
+            # add a blank character to increase space between icon and text
+            item.setText(" " + action.text())
+            w = 60 + self.view.fontMetrics().boundingRect(item.text()).width() + sw
+
+        item.setSizeHint(QSize(w, self.itemHeight))
+        return w
+
+    def _longestShortcutWidth(self):
+        """ longest shortcut key """
+        fm = QFontMetrics(getFont(12))
+        return max(fm.boundingRect(a.shortcut().toString()).width() for a in self.menuActions())
+
     def _createItemIcon(self, w):
         """ create the icon of menu item """
         hasIcon = self._hasItemIcon()
-        icon = QIcon(MenuIconEngine(w.icon()))
+        icon = QIcon(FluentIconEngine(w.icon()))
 
         if hasIcon and w.icon().isNull():
             pixmap = QPixmap(self.view.iconSize())
@@ -310,7 +464,7 @@ class RoundMenu(QWidget):
 
         return icon
 
-    def insertAction(self, before, action):
+    def insertAction(self, before: Union[QAction, Action], action: Union[QAction, Action]):
         """ inserts action to menu, before the action before """
         if before not in self._actions:
             return
@@ -324,7 +478,7 @@ class RoundMenu(QWidget):
         self.view.insertItem(index, item)
         self.adjustSize()
 
-    def addActions(self, actions):
+    def addActions(self, actions: List[Union[QAction, Action]]):
         """ add actions to menu
 
         Parameters
@@ -335,12 +489,12 @@ class RoundMenu(QWidget):
         for action in actions:
             self.addAction(action)
 
-    def insertActions(self, before, actions):
+    def insertActions(self, before: Union[QAction, Action], actions: List[Union[QAction, Action]]):
         """ inserts the actions actions to menu, before the action before """
         for action in actions:
             self.insertAction(before, action)
 
-    def removeAction(self, action):
+    def removeAction(self, action: Union[QAction, Action]):
         """ remove action from menu """
         if action not in self._actions:
             return
@@ -356,7 +510,7 @@ class RoundMenu(QWidget):
         if widget:
             widget.deleteLater()
 
-    def setDefaultAction(self, action):
+    def setDefaultAction(self, action: Union[QAction, Action]):
         """ set the default action """
         if action not in self._actions:
             return
@@ -380,7 +534,7 @@ class RoundMenu(QWidget):
         self.view.setItemWidget(item, w)
         self.adjustSize()
 
-    def insertMenu(self, before, menu):
+    def insertMenu(self, before: Union[QAction, Action], menu):
         """ insert menu before action `before` """
         if not isinstance(menu, RoundMenu):
             raise ValueError('`menu` should be an instance of `RoundMenu`.')
@@ -398,11 +552,11 @@ class RoundMenu(QWidget):
 
         item = QListWidgetItem(self._createItemIcon(menu), menu.title())
         if not self._hasItemIcon():
-            w = 48 + self.view.fontMetrics().boundingRect(menu.title()).width()
+            w = 60 + self.view.fontMetrics().boundingRect(menu.title()).width()
         else:
             # add a blank character to increase space between icon and text
             item.setText(" " + item.text())
-            w = 60 + self.view.fontMetrics().boundingRect(item.text()).width()
+            w = 72 + self.view.fontMetrics().boundingRect(item.text()).width()
 
         # add submenu item
         menu._setParentMenu(self, item)
@@ -439,21 +593,20 @@ class RoundMenu(QWidget):
         m = self.view.viewportMargins()
         w = self.view.width()-m.left()-m.right()
 
-        # icon separator
-        separator = MenuSeparator(self.view)
-        separator.resize(w, separator.height())
-
         # add separator to list widget
         item = QListWidgetItem(self.view)
         item.setFlags(Qt.ItemFlag.NoItemFlags)
-        item.setSizeHint(QSize(w, separator.height()))
+        item.setSizeHint(QSize(w, 9))
         self.view.addItem(item)
-        self.view.setItemWidget(item, separator)
+        item.setData(Qt.ItemDataRole.DecorationRole, "seperator")
         self.adjustSize()
 
     def _onItemClicked(self, item):
-        action = item.data(Qt.ItemDataRole.UserRole)
-        if action not in self._actions:
+        action = item.data(Qt.ItemDataRole.UserRole)  # type: QAction
+        if action not in self._actions or not action.isEnabled():
+            return
+
+        if self.view.itemWidget(item) and not action.property('selectable'):
             return
 
         self._hideMenu(False)
@@ -468,12 +621,10 @@ class RoundMenu(QWidget):
 
     def _closeParentMenu(self):
         menu = self
-        while menu.parentMenu:
+        while menu:
             menu.close()
             menu = menu.parentMenu
 
-        menu.close()
-
     def _onItemEntered(self, item):
         self.lastHoverItem = item
         if not isinstance(item.data(Qt.ItemDataRole.UserRole), RoundMenu):
@@ -499,12 +650,14 @@ class RoundMenu(QWidget):
     def closeEvent(self, e):
         e.accept()
         self.closedSignal.emit()
+        self.view.clearSelection()
 
     def menuActions(self):
         return self._actions
 
     def mousePressEvent(self, e):
-        if self.childAt(e.pos()) is not self.view:
+        w = self.childAt(e.pos())
+        if (w is not self.view) and (not self.view.isAncestorOf(w)):
             self._hideMenu(True)
 
     def mouseMoveEvent(self, e):
@@ -524,57 +677,23 @@ class RoundMenu(QWidget):
             view.clearSelection()
             self._hideMenu(False)
 
-            # update style
-            index = view.row(self.menuItem)
-            if index > 0:
-                view.item(index-1).setFlags(Qt.ItemFlag.ItemIsEnabled)
-            if index < view.count()-1:
-                view.item(index+1).setFlags(Qt.ItemFlag.ItemIsEnabled)
-
     def _onActionChanged(self):
         """ action changed slot """
-        action = self.sender()
-        item = action.property('item')
+        action = self.sender()  # type: QAction
+        item = action.property('item')  # type: QListWidgetItem
         item.setIcon(self._createItemIcon(action))
 
-        if not self._hasItemIcon():
-            item.setText(action.text())
-            w = 28 + self.view.fontMetrics().boundingRect(action.text()).width()
+        self._adjustItemText(item, action)
+
+        if action.isEnabled():
+            item.setFlags(Qt.ItemFlag.ItemIsEnabled | Qt.ItemFlag.ItemIsSelectable)
         else:
-            # add a blank character to increase space between icon and text
-            item.setText(" " + action.text())
-            w = 60 + self.view.fontMetrics().boundingRect(item.text()).width()
+            item.setFlags(Qt.ItemFlag.NoItemFlags)
 
-        item.setSizeHint(QSize(w, self.itemHeight))
         self.view.adjustSize()
         self.adjustSize()
 
-    def _onActionChanged(self):
-        """ action changed slot """
-        action = self.sender()
-        item = action.property('item')
-        item.setIcon(self._createItemIcon(action))
-
-        if not self._hasItemIcon():
-            item.setText(action.text())
-            w = 28 + self.view.fontMetrics().boundingRect(action.text()).width()
-        else:
-            # add a blank character to increase space between icon and text
-            item.setText(" " + action.text())
-            w = 60 + self.view.fontMetrics().boundingRect(item.text()).width()
-
-        item.setSizeHint(QSize(w, self.itemHeight))
-        self.view.adjustSize()
-        self.adjustSize()
-
-    def _onSlideValueChanged(self, pos):
-        m = self.layout().contentsMargins()
-        w = self.view.width() + m.left() + m.right() + 120
-        h = self.view.height() + m.top() + m.bottom() + 20
-        y = self.ani.endValue().y() - pos.y()
-        self.setMask(QRegion(0, y, w, h))
-
-    def exec(self, pos, ani=True):
+    def exec(self, pos, ani=True, aniType=MenuAnimationType.DROP_DOWN):
         """ show menu
 
         Parameters
@@ -584,38 +703,234 @@ class RoundMenu(QWidget):
 
         ani: bool
             Whether to show pop-up animation
+
+        aniType: MenuAnimationType
+            menu animation type
         """
-        if self.isVisible():
-            return
+        #if self.isVisible():
+        #    aniType = MenuAnimationType.NONE
 
-        rect = QApplication.screenAt(QCursor.pos()).availableGeometry()
-        w, h = self.width() + 5, self.height() + 5
-        pos.setX(min(pos.x() - self.layout().contentsMargins().left(), rect.right() - w))
-        pos.setY(min(pos.y() - 4, rect.bottom() - h))
-
-        if ani:
-            self.ani.setStartValue(pos-QPoint(0, int(h/2)))
-            self.ani.setEndValue(pos)
-            self.ani.setDuration(250)
-            self.ani.setEasingCurve(QEasingCurve.Type.OutQuad)
-            self.ani.start()
-        else:
-            self.move(pos)
+        self.aniManager = MenuAnimationManager.make(self, aniType)
+        self.aniManager.exec(pos)
 
         self.show()
 
-        if not self.isSubMenu:
-            return
+        if self.isSubMenu:
+            self.menuItem.setSelected(True)
 
-        self.menuItem.setSelected(True)
+    def adjustPosition(self):
+        m = self.layout().contentsMargins()
+        rect = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        w, h = self.layout().sizeHint().width() + 5, self.layout().sizeHint().height()
 
-        # temporarily disable item to change style
-        view = self.parentMenu.view
-        index = view.row(self.menuItem)
-        if index > 0:
-            view.item(index-1).setFlags(Qt.ItemFlag.NoItemFlags)
-        if index < view.count()-1:
-            view.item(index+1).setFlags(Qt.ItemFlag.NoItemFlags)
+        x = min(self.x() - m.left(), rect.right() - w)
+        y = self.y()
+        if y > rect.bottom() - h:
+            y = self.y() - h + m.bottom()
+
+        self.move(x, y)
+
+
+class MenuAnimationManager(QObject):
+    """ Menu animation manager """
+
+    managers = {}
+
+    def __init__(self, menu: RoundMenu):
+        super().__init__()
+        self.menu = menu
+        self.ani = QPropertyAnimation(menu, b'pos', menu)
+
+        self.ani.setDuration(250)
+        self.ani.setEasingCurve(QEasingCurve.Type.OutQuad)
+        self.ani.valueChanged.connect(self._onValueChanged)
+        self.ani.valueChanged.connect(self._updateMenuViewport)
+
+    def _onValueChanged(self):
+        pass
+
+    def availableViewSize(self, pos: QPoint):
+        """ Return the available size of view """
+        ss = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        w, h = ss.width() - 100, ss.height() - 100
+        return w, h
+
+    def _updateMenuViewport(self):
+        self.menu.view.viewport().update()
+        self.menu.view.setAttribute(Qt.WidgetAttribute.WA_UnderMouse, True)
+        e = QHoverEvent(QEvent.Type.HoverEnter, QPointF(), QPointF(1, 1))
+        QApplication.sendEvent(self.menu.view, e)
+
+    def _endPosition(self, pos):
+        m = self.menu
+        rect = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        w, h = m.width() + 5, m.height()
+        x = min(pos.x() - m.layout().contentsMargins().left(), rect.right() - w)
+        y = min(pos.y() - 4, rect.bottom() - h)
+
+        return QPoint(x, y)
+
+    def _menuSize(self):
+        m = self.menu.layout().contentsMargins()
+        w = self.menu.view.width() + m.left() + m.right() + 120
+        h = self.menu.view.height() + m.top() + m.bottom() + 20
+        return w, h
+
+    def exec(self, pos: QPoint):
+        pass
+
+    @classmethod
+    def register(cls, name):
+        """ register menu animation manager
+
+        Parameters
+        ----------
+        name: Any
+            the name of manager, it should be unique
+        """
+        def wrapper(Manager):
+            if name not in cls.managers:
+                cls.managers[name] = Manager
+
+            return Manager
+
+        return wrapper
+
+    @classmethod
+    def make(cls, menu: RoundMenu, aniType: MenuAnimationType):
+        if aniType not in cls.managers:
+            raise ValueError(f'`{aniType}` is an invalid menu animation type.')
+
+        return cls.managers[aniType](menu)
+
+
+@MenuAnimationManager.register(MenuAnimationType.NONE)
+class DummyMenuAnimationManager(MenuAnimationManager):
+    """ Dummy menu animation manager """
+
+    def exec(self, pos: QPoint):
+        self.menu.move(self._endPosition(pos))
+
+
+@MenuAnimationManager.register(MenuAnimationType.DROP_DOWN)
+class DropDownMenuAnimationManager(MenuAnimationManager):
+    """ Drop down menu animation manager """
+
+    def exec(self, pos):
+        pos = self._endPosition(pos)
+        h = self.menu.height() + 5
+
+        self.ani.setStartValue(pos-QPoint(0, int(h/2)))
+        self.ani.setEndValue(pos)
+        self.ani.start()
+
+    def availableViewSize(self, pos: QPoint):
+        ss = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        return ss.width() - 100, max(ss.bottom() - pos.y() - 10, 1)
+
+    def _onValueChanged(self):
+        w, h = self._menuSize()
+        y = self.ani.endValue().y() - self.ani.currentValue().y()
+        self.menu.setMask(QRegion(0, y, w, h))
+
+
+@MenuAnimationManager.register(MenuAnimationType.PULL_UP)
+class PullUpMenuAnimationManager(MenuAnimationManager):
+    """ Pull up menu animation manager """
+
+    def _endPosition(self, pos):
+        m = self.menu
+        rect = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        w, h = m.width() + 5, m.height()
+        x = min(pos.x() - m.layout().contentsMargins().left(), rect.right() - w)
+        y = max(pos.y() - h + 13, 4)
+        return QPoint(x, y)
+
+    def exec(self, pos):
+        pos = self._endPosition(pos)
+        h = self.menu.height() + 5
+
+        self.ani.setStartValue(pos+QPoint(0, int(h/2)))
+        self.ani.setEndValue(pos)
+        self.ani.start()
+
+    def availableViewSize(self, pos: QPoint):
+        ss = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        return ss.width() - 100, max(pos.y() - 28, 1)
+
+    def _onValueChanged(self):
+        w, h = self._menuSize()
+        y = self.ani.endValue().y() - self.ani.currentValue().y()
+        self.menu.setMask(QRegion(0, y, w, h - 28))
+
+
+@MenuAnimationManager.register(MenuAnimationType.FADE_IN_DROP_DOWN)
+class FadeInDropDownMenuAnimationManager(MenuAnimationManager):
+    """ Fade in drop down menu animation manager """
+
+    def __init__(self, menu: RoundMenu):
+        super().__init__(menu)
+        self.opacityAni = QPropertyAnimation(menu, b'windowOpacity', self)
+        self.aniGroup = QParallelAnimationGroup(self)
+        self.aniGroup.addAnimation(self.ani)
+        self.aniGroup.addAnimation(self.opacityAni)
+
+    def exec(self, pos):
+        pos = self._endPosition(pos)
+
+        self.opacityAni.setStartValue(0)
+        self.opacityAni.setEndValue(1)
+        self.opacityAni.setDuration(150)
+        self.opacityAni.setEasingCurve(QEasingCurve.Type.OutQuad)
+
+        self.ani.setStartValue(pos-QPoint(0, 8))
+        self.ani.setEndValue(pos)
+        self.ani.setDuration(150)
+        self.ani.setEasingCurve(QEasingCurve.Type.OutQuad)
+
+        self.aniGroup.start()
+
+    def availableViewSize(self, pos: QPoint):
+        ss = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        return ss.width() - 100, max(ss.bottom() - pos.y() - 10, 1)
+
+
+@MenuAnimationManager.register(MenuAnimationType.FADE_IN_PULL_UP)
+class FadeInPullUpMenuAnimationManager(MenuAnimationManager):
+    """ Fade in pull up menu animation manager """
+
+    def __init__(self, menu: RoundMenu):
+        super().__init__(menu)
+        self.opacityAni = QPropertyAnimation(menu, b'windowOpacity', self)
+        self.aniGroup = QParallelAnimationGroup(self)
+        self.aniGroup.addAnimation(self.ani)
+        self.aniGroup.addAnimation(self.opacityAni)
+
+    def _endPosition(self, pos):
+        m = self.menu
+        rect = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        w, h = m.width() + 5, m.height()
+        x = min(pos.x() - m.layout().contentsMargins().left(), rect.right() - w)
+        y = max(pos.y() - h + 15, 4)
+        return QPoint(x, y)
+
+    def exec(self, pos):
+        pos = self._endPosition(pos)
+
+        self.opacityAni.setStartValue(0)
+        self.opacityAni.setEndValue(1)
+        self.opacityAni.setDuration(150)
+        self.opacityAni.setEasingCurve(QEasingCurve.Type.OutQuad)
+
+        self.ani.setStartValue(pos+QPoint(0, 8))
+        self.ani.setEndValue(pos)
+        self.ani.setDuration(200)
+        self.ani.setEasingCurve(QEasingCurve.Type.OutQuad)
+        self.aniGroup.start()
+
+    def availableViewSize(self, pos: QPoint):
+        ss = QApplication.screenAt(QCursor.pos()).availableGeometry()
+        return ss.width() - 100, pos.y() - 28
 
 
 class EditMenu(RoundMenu):
@@ -667,29 +982,43 @@ class EditMenu(RoundMenu):
     def _parentSelectedText(self):
         raise NotImplementedError
 
-    def exec(self, pos, ani=True):
+    def exec(self, pos, ani=True, aniType=MenuAnimationType.DROP_DOWN):
         self.clear()
         self.createActions()
 
         if QApplication.clipboard().mimeData().hasText():
             if self._parentText():
                 if self._parentSelectedText():
-                    self.addActions(self.action_list)
+                    if self.parent().isReadOnly():
+                        self.addActions([self.copyAct, self.selectAllAct])
+                    else:
+                        self.addActions(self.action_list)
                 else:
-                    self.addActions(self.action_list[2:])
-            else:
+                    if self.parent().isReadOnly():
+                        self.addAction(self.selectAllAct)
+                    else:
+                        self.addActions(self.action_list[2:])
+            elif not self.parent().isReadOnly():
                 self.addAction(self.pasteAct)
-        else:
-            if self._parentText():
-                if self._parentSelectedText():
-                    self.addActions(
-                        self.action_list[:2] + self.action_list[3:])
-                else:
-                    self.addActions(self.action_list[3:])
             else:
                 return
+        else:
+            if not self._parentText():
+                return
 
-        super().exec(pos, ani)
+            if self._parentSelectedText():
+                if self.parent().isReadOnly():
+                    self.addAction([self.copyAct, self.selectAllAct])
+                else:
+                    self.addActions(
+                        self.action_list[:2] + self.action_list[3:])
+            else:
+                if self.parent().isReadOnly():
+                    self.addAction(self.selectAllAct)
+                else:
+                    self.addActions(self.action_list[3:])
+
+        super().exec(pos, ani, aniType)
 
 
 class LineEditMenu(EditMenu):
@@ -736,3 +1065,123 @@ class TextEditMenu(EditMenu):
                 QTextCursor.MoveOperation.Right, QTextCursor.MoveMode.KeepAnchor, self.selectionLength)
 
         super()._onItemClicked(item)
+
+
+class IndicatorMenuItemDelegate(MenuItemDelegate):
+    """ Menu item delegate with indicator """
+
+    def paint(self, painter: QPainter, option, index):
+        super().paint(painter, option, index)
+        if not option.state & QStyle.StateFlag.State_Selected:
+            return
+
+        painter.save()
+        painter.setRenderHints(
+            QPainter.RenderHint.Antialiasing |
+            QPainter.RenderHint.SmoothPixmapTransform |
+            QPainter.RenderHint.TextAntialiasing
+        )
+
+        painter.setPen(Qt.PenStyle.NoPen)
+        painter.setBrush(themeColor())
+        painter.drawRoundedRect(6, 11+option.rect.y(), 3, 15, 1.5, 1.5)
+
+        painter.restore()
+
+
+class CheckableMenuItemDelegate(ShortcutMenuItemDelegate):
+    """ Checkable menu item delegate """
+
+    def _drawIndicator(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
+        raise NotImplementedError
+
+    def paint(self, painter: QPainter, option: QStyleOptionViewItem, index: QModelIndex):
+        super().paint(painter, option, index)
+
+        # draw indicator
+        action = index.data(Qt.ItemDataRole.UserRole)  # type: QAction
+        if not (isinstance(action, QAction) and action.isChecked()):
+            return
+
+        painter.save()
+        self._drawIndicator(painter, option, index)
+        painter.restore()
+
+
+class RadioIndicatorMenuItemDelegate(CheckableMenuItemDelegate):
+    """ Checkable menu item delegate with radio indicator """
+
+    def _drawIndicator(self, painter, option, index):
+        rect = option.rect
+        r = 5
+        x = rect.x() + 22
+        y = rect.center().y() - r / 2
+
+        painter.setRenderHints(QPainter.RenderHint.Antialiasing)
+        if not option.state & QStyle.StateFlag.State_MouseOver:
+            painter.setOpacity(0.75 if isDarkTheme() else 0.65)
+
+        painter.setPen(Qt.PenStyle.NoPen)
+        painter.setBrush(Qt.GlobalColor.white if isDarkTheme() else Qt.GlobalColor.black)
+        painter.drawEllipse(QRectF(x, y, r, r))
+
+
+class CheckIndicatorMenuItemDelegate(CheckableMenuItemDelegate):
+    """ Checkable menu item delegate with check indicator """
+
+    def _drawIndicator(self, painter, option, index):
+        rect = option.rect
+        s = 11
+        x = rect.x() + 19
+        y = rect.center().y() - s / 2
+
+        painter.setRenderHints(QPainter.RenderHint.Antialiasing)
+        if not option.state & QStyle.StateFlag.State_MouseOver:
+            painter.setOpacity(0.75)
+
+        FIF.ACCEPT.render(painter, QRectF(x, y, s, s))
+
+
+class MenuIndicatorType(Enum):
+    """ Menu indicator type """
+    CHECK = 0
+    RADIO = 1
+
+
+def createCheckableMenuItemDelegate(style: MenuIndicatorType):
+    """ create checkable menu item delegate """
+    if style == MenuIndicatorType.RADIO:
+        return RadioIndicatorMenuItemDelegate()
+    if style == MenuIndicatorType.CHECK:
+        return CheckIndicatorMenuItemDelegate()
+
+    raise ValueError(f'`{style}` is not a valid menu indicator type.')
+
+
+class CheckableMenu(RoundMenu):
+    """ Checkable menu """
+
+    def __init__(self, title="", parent=None, indicatorType=MenuIndicatorType.CHECK):
+        super().__init__(title, parent)
+        self.view.setItemDelegate(createCheckableMenuItemDelegate(indicatorType))
+        self.view.setObjectName('checkableListWidget')
+
+    def _adjustItemText(self, item: QListWidgetItem, action: QAction):
+        w = super()._adjustItemText(item, action)
+        item.setSizeHint(QSize(w + 26, self.itemHeight))
+
+
+class SystemTrayMenu(RoundMenu):
+    """ System tray menu """
+
+    def showEvent(self, e):
+        super().showEvent(e)
+        self.adjustPosition()
+
+
+class CheckableSystemTrayMenu(CheckableMenu):
+    """ Checkable system tray menu """
+
+    def showEvent(self, e):
+        super().showEvent(e)
+        self.adjustPosition()
