@@ -8,6 +8,7 @@ from PyQt6.QtWidgets import (QSpinBox, QDoubleSpinBox, QToolButton, QHBoxLayout,
 
 from ...common.style_sheet import FluentStyleSheet, themeColor
 from ...common.icon import FluentIconBase, Theme, getIconColor
+from ...common.font import setFont
 from ...components.widgets import LineEditMenu
 
 
@@ -18,12 +19,7 @@ class SpinIcon(FluentIconBase, Enum):
     DOWN = "Down"
 
     def path(self, theme=Theme.AUTO):
-        if theme == Theme.AUTO:
-            c = getIconColor()
-        else:
-            c = "white" if theme == Theme.DARK else "black"
-
-        return f':/qfluentwidgets/images/spin_box/{self.value}_{c}.svg'
+        return f':/qfluentwidgets/images/spin_box/{self.value}_{getIconColor(theme)}.svg'
 
 
 
@@ -31,30 +27,43 @@ class SpinButton(QToolButton):
 
     def __init__(self, icon: SpinIcon, parent=None):
         super().__init__(parent=parent)
+        self.isPressed = False
         self._icon = icon
         self.setFixedSize(31, 23)
         self.setIconSize(QSize(10, 10))
         FluentStyleSheet.SPIN_BOX.apply(self)
 
+    def mousePressEvent(self, e):
+        self.isPressed = True
+        super().mousePressEvent(e)
+
+    def mouseReleaseEvent(self, e):
+        self.isPressed = False
+        super().mouseReleaseEvent(e)
+
     def paintEvent(self, e):
         super().paintEvent(e)
         painter = QPainter(self)
         painter.setRenderHints(QPainter.RenderHint.Antialiasing |
                                QPainter.RenderHint.SmoothPixmapTransform)
 
+        if self.isPressed:
+            painter.setOpacity(0.7)
+
         self._icon.render(painter, QRectF(10, 9, 11, 11))
 
 
-class Ui_SpinBox:
+class SpinBoxBase:
     """ Spin box ui """
 
-    def __init__(self, *args, **kwargs):
-        pass
+    def __init__(self, parent=None):
+        super().__init__(parent=parent)
 
-    def _setUpUi(self):
+        self.setProperty('transparent', True)
         FluentStyleSheet.SPIN_BOX.apply(self)
         self.setButtonSymbols(QSpinBox.ButtonSymbols.NoButtons)
         self.setFixedHeight(33)
+        setFont(self)
 
         self.hBoxLayout = QHBoxLayout(self)
         self.upButton = SpinButton(SpinIcon.UP, self)
@@ -77,6 +86,11 @@ class Ui_SpinBox:
         menu = LineEditMenu(self.lineEdit())
         menu.exec(self.mapToGlobal(pos))
 
+    def setAccelerated(self, on: bool):
+        super().setAccelerated(on)
+        self.upButton.setAutoRepeat(on)
+        self.downButton.setAutoRepeat(on)
+
     def _drawBorderBottom(self):
         if not self.hasFocus():
             return
@@ -96,62 +110,42 @@ class Ui_SpinBox:
         painter.fillPath(path, themeColor())
 
 
-class SpinBox(QSpinBox, Ui_SpinBox):
+class SpinBox(SpinBoxBase, QSpinBox):
     """ Spin box """
 
-    def __init__(self, parent=None):
-        super().__init__(parent=parent)
-        self._setUpUi()
-
     def paintEvent(self, e):
-        super().paintEvent(e)
+        QSpinBox.paintEvent(self, e)
         self._drawBorderBottom()
 
 
-class DoubleSpinBox(QDoubleSpinBox, Ui_SpinBox):
+class DoubleSpinBox(SpinBoxBase, QDoubleSpinBox):
     """ Double spin box """
 
-    def __init__(self, parent=None):
-        super().__init__(parent)
-        self._setUpUi()
-
     def paintEvent(self, e):
-        super().paintEvent(e)
+        QDoubleSpinBox.paintEvent(self, e)
         self._drawBorderBottom()
 
 
-class TimeEdit(QTimeEdit, Ui_SpinBox):
+class TimeEdit(SpinBoxBase, QTimeEdit):
     """ Time edit """
 
-    def __init__(self, parent=None):
-        super().__init__(parent)
-        self._setUpUi()
-
     def paintEvent(self, e):
-        super().paintEvent(e)
+        QTimeEdit.paintEvent(self, e)
         self._drawBorderBottom()
 
 
-class DateTimeEdit(QDateTimeEdit, Ui_SpinBox):
+class DateTimeEdit(SpinBoxBase, QDateTimeEdit):
     """ Date time edit """
 
-    def __init__(self, parent=None):
-        super().__init__(parent)
-        self._setUpUi()
-
     def paintEvent(self, e):
-        super().paintEvent(e)
+        QDateTimeEdit.paintEvent(self, e)
         self._drawBorderBottom()
 
 
-class DateEdit(QDateEdit, Ui_SpinBox):
+class DateEdit(SpinBoxBase, QDateEdit):
     """ Date edit """
 
-    def __init__(self, parent=None):
-        super().__init__(parent)
-        self._setUpUi()
-
     def paintEvent(self, e):
-        super().paintEvent(e)
+        QDateEdit.paintEvent(self, e)
         self._drawBorderBottom()
 
